@@ -29,6 +29,7 @@ class Builtin:
 
         self.ctx.register_attr(FunctionType)
         self.ctx.register_attr(Float32Type)
+        self.ctx.register_attr(Float64Type)
         self.ctx.register_attr(IntegerType)
         self.ctx.register_attr(IndexType)
 
@@ -112,7 +113,7 @@ class IntegerType(ParametrizedAttribute):
 
     @staticmethod
     @builder
-    def from_width(width: int) -> Attribute:
+    def from_width(width: int) -> IntegerType:
         return IntegerType([IntAttr.from_int(width)])
 
 
@@ -382,6 +383,13 @@ class Float32Type(ParametrizedAttribute):
 f32 = Float32Type()
 
 
+class Float64Type(ParametrizedAttribute):
+    name = "f64"
+
+
+f64 = Float64Type()
+
+
 @irdl_attr_definition
 class UnitAttr(ParametrizedAttribute):
     name: str = "unit"
