@@ -1,4 +1,4 @@
-from typing import Tuple
+from typing import Tuple, Union
 
 from xdsl.ir import *
 
@@ -20,7 +20,7 @@ class Rewriter:
 
     @staticmethod
     def replace_op(op: Operation,
-                   new_ops: Union[Operation, List[Operation]],
+                   new_ops: Operation | List[Operation],
                    new_results: Optional[List[Optional[SSAValue]]] = None,
                    safe_erase: bool = True):
         """
@@ -107,7 +107,7 @@ class Rewriter:
         Rewriter.inline_block_at_pos(block, op_block, op_pos + 1)
 
     @staticmethod
-    def insert_block_after(block: Union[Block, List[Block]], target: Block):
+    def insert_block_after(block: Block | List[Block], target: Block):
         """
         Insert one or multiple blocks after another block.
         The blocks to insert should be detached from any region.
@@ -123,7 +123,7 @@ class Rewriter:
         region.insert_block(block_list, pos + 1)
 
     @staticmethod
-    def insert_block_before(block: Union[Block, List[Block]], target: Block):
+    def insert_block_before(block: Block | List[Block], target: Block):
         """
         Insert one or multiple block before another block.
         The blocks to insert should be detached from any region.
