@@ -11,7 +11,10 @@ from xdsl.dialects.arith import *
 from xdsl.dialects.affine import *
 from xdsl.dialects.memref import *
 from xdsl.dialects.builtin import *
+from xdsl.dialects.cmath import *
 from xdsl.dialects.cf import *
+from xdsl.dialects.irdl import *
+from xdsl.dialects.llvm import LLVM
 
 
 class xDSLOptMain:
@@ -39,7 +42,7 @@ class xDSLOptMain:
     stream.
     """
 
-    pipeline: List[Tuple[str, Callable[[ModuleOp], None]]]
+    pipeline: List[tuple[str, Callable[[ModuleOp], None]]]
     """ The pass-pipeline to be applied. """
 
     def __init__(self, description='xDSL modular optimizer driver'):
@@ -147,6 +150,9 @@ class xDSLOptMain:
         affine = Affine(self.ctx)
         scf = Scf(self.ctx)
         cf = Cf(self.ctx)
+        cmath = CMath(self.ctx)
+        irdl = IRDL(self.ctx)
+        llvm = LLVM(self.ctx)
 
     def register_all_frontends(self):
         """
@@ -225,7 +231,7 @@ class xDSLOptMain:
             f = sys.stdin
             file_extension = 'xdsl'
         else:
-            f = open(self.args.input_file, mode='r')
+            f = open(self.args.input_file)
             _, file_extension = os.path.splitext(self.args.input_file)
             file_extension = file_extension.replace(".", "")
 
